@@ -14,12 +14,15 @@ use Plack::Util::Accessor qw/
     dot_png
     font
     text
+    filter
+    cache
+    cache_key
     max_width
     max_height
     stderr
 /;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 sub prepare_app {
     my $self = shift;
@@ -79,6 +82,16 @@ sub call {
         my $line   = int($req->param('line') || 1); $line++;
         return $self->return_status(400) if $line > $w && $line > $h;
 
+        if ($self->cache) {
+            $self->cache_key(
+                join ':',
+                    $w, $h, $ext, $fill, $border, $line
+            );
+            if ( my $cache = $self->cache->get($self->cache_key) ) {
+                return [ 200, @{$cache} ];
+            }
+        }
+
         my $img = Imager->new(xsize => $w, ysize => $h);
         $img->box(
             filled => 1,
@@ -111,12 +124,16 @@ sub call {
             }
         }
 
+        if (ref($self->filter) eq 'CODE') {
+            $self->filter->($self, $img);
+        }
+
         my $content = '';
         $img->write(data => \$content , type => $ext);
         my $disposition = $ext_obj->disposition. '; filename="'
                             . "${w}x$h\.$ext". '"';
-        return [
-            200,
+
+        my $response = [
             [
                 'Content-Type'   => $ext_obj->type,
                 'Content-Length' => length $content,
@@ -125,6 +142,11 @@ sub call {
             ],
             [$content]
         ];
+
+        if ($self->cache) {
+            $self->cache->set($self->cache_key => $response);
+        }
+        return [ 200, @{$response} ];
     }
 }
 
@@ -221,6 +243,11 @@ size of border line(pixel): default 1
             color => 'red', # option
         },
         text   => "foo",
+        filter => sub {
+            my ($self, $img) = @_;
+            # .. do something ..
+        },
+        cache => Cache::File->new(cache_root => '/tmp/cache'),
         stderr => 1,
     )->to_app;
 
@@ -238,6 +265,14 @@ If you want to see image size as text on the image, you should set C<font> optio
 
 add a text in the image. C<text> option also requires C<font> option. Note that text string should be decoded utf8 text when it included not ascii strings.
 
+=item filter
+
+filter should code reference. This method receives the $self and Imager object.
+
+=item cache
+
+If you want to cache responses between requests, provide the C<cache> parameter with an object supporting the Cache API(e.g. Cache::File). Specifically, an object that supports $cache->get($key) and $cache->set($key, $value, $expires).
+
 =item stderr
 
 print error message to STDERR when some error was happen.
